namespace PkmnLib.Dynamic.Models;

public record struct BattleResult
{
    private BattleResult(bool conclusiveResult, byte? winningSide)
    {
        ConclusiveResult = conclusiveResult;
        WinningSide = winningSide;
    }
    
    public static BattleResult Inconclusive => new(false, null);
    public static BattleResult Conclusive(byte winningSide) => new(true, winningSide);
    
    /// <summary>
    /// Whether the battle has a conclusive result. If false, no side has won.
    /// </summary>
    public bool ConclusiveResult { get; }
    
    /// <summary>
    /// The side that won the battle. If null, no side has won.
    /// </summary>
    public byte? WinningSide { get; }
}