using JetBrains.Annotations;

namespace PkmnLib.Dynamic.ScriptHandling.Registry;

/// <summary>
/// A plugin is a way to register scripts and other dynamic components to the script registry.
/// </summary>
[UsedImplicitly(ImplicitUseTargetFlags.WithInheritors)]
public abstract class Plugin
{
    /// <summary>
    /// The name of the plugin. Mostly used for debugging purposes.
    /// </summary> 
    public abstract string Name { get; }

    /// <summary>
    /// When the plugin should be loaded. Lower values are loaded first.
    /// 0 should be reserved for the core battle scripts.
    /// </summary>
    public abstract uint LoadOrder { get; }

    /// <summary>
    /// Run the registration of the plugin when we're building the library.
    /// </summary>
    public abstract void Register(ScriptRegistry registry);
}